# Copyright 2021 DeepMind Technologies Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pyformat: disable
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# pylint: skip-file
# source: xm_local/storage/data.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='xm_local/storage/data.proto',
  package='xmanager',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1bxm_local/storage/data.proto\x12\x08xmanager\"\x8a\x01\n\x03Job\x12#\n\x05local\x18\x01 \x01(\x0b\x32\x12.xmanager.LocalJobH\x00\x12\'\n\x04\x63\x61ip\x18\x02 \x01(\x0b\x32\x17.xmanager.AIPlatformJobH\x00\x12-\n\nkubernetes\x18\x03 \x01(\x0b\x32\x17.xmanager.KubernetesJobH\x00\x42\x06\n\x04kind\"7\n\x08LocalJob\x12\x0b\n\x03pid\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\"&\n\rAIPlatformJob\x12\x15\n\rresource_name\x18\x01 \x01(\t\"4\n\rKubernetesJob\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x10\n\x08job_name\x18\x02 \x01(\tb\x06proto3'
)




_JOB = _descriptor.Descriptor(
  name='Job',
  full_name='xmanager.Job',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='local', full_name='xmanager.Job.local', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='caip', full_name='xmanager.Job.caip', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kubernetes', full_name='xmanager.Job.kubernetes', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='kind', full_name='xmanager.Job.kind',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=42,
  serialized_end=180,
)


_LOCALJOB = _descriptor.Descriptor(
  name='LocalJob',
  full_name='xmanager.LocalJob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pid', full_name='xmanager.LocalJob.pid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cmd', full_name='xmanager.LocalJob.cmd', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='xmanager.LocalJob.timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=182,
  serialized_end=237,
)


_AIPLATFORMJOB = _descriptor.Descriptor(
  name='AIPlatformJob',
  full_name='xmanager.AIPlatformJob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource_name', full_name='xmanager.AIPlatformJob.resource_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=239,
  serialized_end=277,
)


_KUBERNETESJOB = _descriptor.Descriptor(
  name='KubernetesJob',
  full_name='xmanager.KubernetesJob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='xmanager.KubernetesJob.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_name', full_name='xmanager.KubernetesJob.job_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=279,
  serialized_end=331,
)

_JOB.fields_by_name['local'].message_type = _LOCALJOB
_JOB.fields_by_name['caip'].message_type = _AIPLATFORMJOB
_JOB.fields_by_name['kubernetes'].message_type = _KUBERNETESJOB
_JOB.oneofs_by_name['kind'].fields.append(
  _JOB.fields_by_name['local'])
_JOB.fields_by_name['local'].containing_oneof = _JOB.oneofs_by_name['kind']
_JOB.oneofs_by_name['kind'].fields.append(
  _JOB.fields_by_name['caip'])
_JOB.fields_by_name['caip'].containing_oneof = _JOB.oneofs_by_name['kind']
_JOB.oneofs_by_name['kind'].fields.append(
  _JOB.fields_by_name['kubernetes'])
_JOB.fields_by_name['kubernetes'].containing_oneof = _JOB.oneofs_by_name['kind']
DESCRIPTOR.message_types_by_name['Job'] = _JOB
DESCRIPTOR.message_types_by_name['LocalJob'] = _LOCALJOB
DESCRIPTOR.message_types_by_name['AIPlatformJob'] = _AIPLATFORMJOB
DESCRIPTOR.message_types_by_name['KubernetesJob'] = _KUBERNETESJOB
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Job = _reflection.GeneratedProtocolMessageType('Job', (_message.Message,), {
  'DESCRIPTOR' : _JOB,
  '__module__' : 'xm_local.storage.data_pb2'
  # @@protoc_insertion_point(class_scope:xmanager.Job)
  })
_sym_db.RegisterMessage(Job)

LocalJob = _reflection.GeneratedProtocolMessageType('LocalJob', (_message.Message,), {
  'DESCRIPTOR' : _LOCALJOB,
  '__module__' : 'xm_local.storage.data_pb2'
  # @@protoc_insertion_point(class_scope:xmanager.LocalJob)
  })
_sym_db.RegisterMessage(LocalJob)

AIPlatformJob = _reflection.GeneratedProtocolMessageType('AIPlatformJob', (_message.Message,), {
  'DESCRIPTOR' : _AIPLATFORMJOB,
  '__module__' : 'xm_local.storage.data_pb2'
  # @@protoc_insertion_point(class_scope:xmanager.AIPlatformJob)
  })
_sym_db.RegisterMessage(AIPlatformJob)

KubernetesJob = _reflection.GeneratedProtocolMessageType('KubernetesJob', (_message.Message,), {
  'DESCRIPTOR' : _KUBERNETESJOB,
  '__module__' : 'xm_local.storage.data_pb2'
  # @@protoc_insertion_point(class_scope:xmanager.KubernetesJob)
  })
_sym_db.RegisterMessage(KubernetesJob)


# @@protoc_insertion_point(module_scope)
