# Copyright 2021 DeepMind Technologies Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helper module for using TPUs."""
from typing import List


# pylint: disable=line-too-long
def tpuvm_docker_instructions() -> List[str]:
  return [
      'RUN wget '
      'https://storage.googleapis.com/cloud-tpu-tpuvm-artifacts/libtpu/20210525/libtpu.so'
      ' -O /lib/libtpu.so',
      'RUN chmod 700 /lib/libtpu.so',
      'RUN wget '
      'https://storage.googleapis.com/cloud-tpu-tpuvm-artifacts/tensorflow/20210525/tf_nightly-2.6.0-cp38-cp38-linux_x86_64.whl',
      'RUN pip3 install tf_nightly-2.6.0-cp38-cp38-linux_x86_64.whl',
      'RUN rm tf_nightly-2.6.0-cp38-cp38-linux_x86_64.whl',
  ]
