# Copyright 2021 DeepMind Technologies Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""XManager client API.

Provides XManager public API for configuring and launching experiments.
"""

from xmanager.xm import job_operators
from xmanager.xm.compute_units import *
from xmanager.xm.core import *
from xmanager.xm.executables import *
from xmanager.xm.job_blocks import *
from xmanager.xm.metadata_context import ContextAnnotations
from xmanager.xm.metadata_context import MetadataContext
from xmanager.xm.packagables import *
from xmanager.xm.resources import *
from xmanager.xm.utils import run_in_asyncio_loop
from xmanager.xm.utils import ShellSafeArg
